@@ -2,18 +2,20 @@ package SQL::Executor;
 use parent qw(Exporter);
 use strict;
 use warnings;
-our $VERSION = '0.13';
+our $VERSION = '0.14';
 
 our @EXPORT_OK = qw(named_bind);
 
 use Class::Accessor::Lite (
-    ro => ['builder', 'dbh', 'allow_empty_condition', 'backup_callback', 'check_empty_bind'],
+    ro => ['builder', 'handler', 'allow_empty_condition', 'backup_callback', 'check_empty_bind'],
     rw => ['callback'],
 );
 use SQL::Maker;
 use Carp qw();
 use Try::Tiny;
 use SQL::Executor::Iterator;
+use DBIx::Handler;
+
 
 =head1 NAME
 
@@ -84,17 +86,62 @@ These callbacks are useful for making row object.
 sub new {
     my ($class, $dbh, $option_href) = @_;
     my $builder = SQL::Maker->new( driver => $dbh->{Driver}->{Name} );
-    SQL::Maker->load_plugin('InsertMulti');
 
     my $self = {
-        builder               => $builder,
-        dbh                   => $dbh,
+        builder => $builder,
+        dbh     => $dbh,
+        _options($option_href),
+    };
+    bless $self, $class;
+}
+
+=head2 connect($dsn, $user, $pass, $option_for_dbi, $option_href)
+
+$dsn: DSN
+$user: database user
+$pass: database password
+$option_href_for_dbi: options passed to DBI
+$option_href: option for SQL::Executor (options are same as new() method)
+
+connect database and create SQL::Executor instance. using this method, SQL::Executor uses
+managed connection and transaction via L<DBIx::Handler>
+
+=cut
+
+sub connect {
+    my ($class, $dsn, $user, $pass, $option_href_for_dbi, $option_href) = @_;
+    my $handler = DBIx::Handler->new($dsn, $user, $pass, $option_href_for_dbi);
+    my $builder = SQL::Maker->new( driver => $handler->dbh->{Driver}->{Name} );
+
+    my $self = {
+        builder => $builder,
+        handler => $handler,
+        _options($option_href),
+    };
+    bless $self, $class;
+}
+
+sub _options {
+    my ($option_href) = @_;
+
+    return (
         allow_empty_condition => defined $option_href->{allow_empty_condition} ? $option_href->{allow_empty_condition} : 1,
         check_empty_bind      => !!$option_href->{check_empty_bind},
         callback              => $option_href->{callback},
         backup_callback       => $option_href->{callback},
-    };
-    bless $self, $class;
+    );
+}
+
+=head2 dbh()
+
+return database handler
+
+=cut
+
+sub dbh {
+    my ($self) = @_;
+    return $self->handler->dbh if ( defined $self->handler );
+    return $self->{dbh};
 }
 
 
@@ -491,11 +538,30 @@ Do INSERT-multi statement using L<SQL::Maker::Plugin::InsertMulti>.
 
 sub insert_multi {
     my ($self, $table_name, @args) = @_;
+    $self->_load_sql_maker_plugin('InsertMulti');
     my $builder = $self->builder;
     my ($sql, @binds) = $builder->insert_multi($table_name, @args);
     $self->_execute_and_finish($sql, \@binds);
 }
 
+=head2 insert_on_duplicate($table_name, $insert_value_href, $update_value_href)
+
+Do "INSERT ... ON DUPLICATE KEY UPDATE" query (works only MySQL) using L<SQL::Maker::Plugin::InsertOnDuplicate>.
+
+this method is available when L<SQL::Maker> >= 1.09 is installed. If older version is installed, you will
+got error like "Can't locate SQL/Maker/Plugin/InsertOnDuplicate.pm in @INC ..."
+
+=cut
+
+sub insert_on_duplicate {
+    my ($self, $table_name, $insert_value_href, $update_value_href) = @_;
+    $self->_load_sql_maker_plugin('InsertOnDuplicate');
+    my $builder = $self->builder;
+    my ($sql, @binds) = $builder->insert_on_duplicate($table_name, $insert_value_href, $update_value_href);
+    $self->_execute_and_finish($sql, \@binds);
+}
+
+
 
 =head2 delete($table_name, $where)
 
@@ -643,13 +709,35 @@ sub select_id {
     return;
 }
 
+# load SQL::Maker plugin
+sub _load_sql_maker_plugin {
+    my ($self, $plugin_name) = @_;
+
+    if( !defined $self->{sql_maker_load_plugin}->{$plugin_name} ) {
+        SQL::Maker->load_plugin($plugin_name);
+        $self->{sql_maker_load_plugin}->{$plugin_name} = 1;
+    }
+}
+
+
 1;
 __END__
 
 
 =head1 How to use Transaction.
 
-You can use L<DBI>'s transaction (begin_work and commit).
+When create instance using connect() method, you can use L<DBIx::Handler>'s
+transaction management, 
+
+  use SQL::Executor;
+  my $ex = SQL::Executor->connect($dsn, $id, $pass);
+  my $txn = $ex->handler->txn_scope();
+  $ex->insert('SOME_TABLE', { id => 124, value => 'xxxx'} );
+  $ex->insert('SOME_TABLE', { id => 125, value => 'yyy'} );
+  $txn->commit();
+
+
+Or You can use L<DBI>'s transaction (begin_work and commit).
 
   use DBI;
   use SQL::Executor;
@@ -684,8 +772,6 @@ Or you can also use transaction management modules like L<DBIx::TransactionManag
 
 =item * When I need to use complex query, I want to use named placeholder.
 
-=item * I don't want to manage transaction in this module.
-
 =back
 
 =head1 AUTHOR
